﻿using Borepin.ViewModel;
using Prism.Mvvm;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Input;
using Prism.Commands;
using Prism.Navigation;
using Borepin.Service.Connections;
using Borepin.Service.BFFH;
using Borepin.Model;

namespace Borepin.PageModel
{
    public class MachineListPageModel : BindableBase, INavigationAware
    {
        private readonly INavigationService _NavigationService;
        private readonly IConnectionService _ConnectionService;
        private readonly IBFFHService _BFFHService;

        public MachineListPageModel(INavigationService navigationService, IConnectionService connectionService, IBFFHService bffhService)
        {
            _NavigationService = navigationService;
            _ConnectionService = connectionService;
            _BFFHService = bffhService;

            SelectInstanceCommand = new DelegateCommand<object>(SelectInstanceCommandExecuted);
        }

        private async Task<bool> LoadData()
        {
            if(_BFFHService.ActiveConnection == null)
            {
                return await Task.FromResult(true);
            }

            FabAccessAPI.Machines machineInterface = await _BFFHService.GetMachineInterface();

            List<Machine> list = new List<Machine>();

            IReadOnlyList<FabAccessAPI.Machine> machine_list = await machineInterface.ListMachines();
            foreach(FabAccessAPI.Machine machine in machine_list)
            {
                list.Add(new Machine() { Instance = machine, MInfo = await machine.GetMInfo() });
            }

            List<MachineListItemViewModel> viewmodel_list = new List<MachineListItemViewModel>();
            foreach(Machine machine in list)
            {
                viewmodel_list.Add(new MachineListItemViewModel(machine));
            }

            MachineListItemViewModel_List = viewmodel_list;

            return await Task.FromResult(true);
        }

        #region Properties
        private List<MachineListItemViewModel> _MachineListItemViewModel_List;
        public List<MachineListItemViewModel> MachineListItemViewModel_List
        {
            get => _MachineListItemViewModel_List;
            set => SetProperty(ref _MachineListItemViewModel_List, value);
        }
        #endregion

        #region Commands
        private ICommand _SelectInstanceCommand;
        public ICommand SelectInstanceCommand
        {
            get => _SelectInstanceCommand;
            set => SetProperty(ref _SelectInstanceCommand, value);
        }
        private async void SelectInstanceCommandExecuted(object obj)
        {
            MachineListItemViewModel viewmodel = obj as MachineListItemViewModel;

            NavigationParameters parameters = new NavigationParameters
            {
                { "instance", viewmodel.Instance }
            };

            INavigationResult result = await _NavigationService.NavigateAsync($"MachinePage", parameters);
            if (!result.Success)
            {
                System.Diagnostics.Debugger.Break();
            }
        }
        #endregion

        #region INavigationAware
        public void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public void OnNavigatedTo(INavigationParameters parameters)
        {
            Task.Run(LoadData);
        }
        #endregion
    }
}
