﻿using Borepin.Model;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Navigation;
using System.Windows.Input;
using static FabAccessAPI.Schema.Machine.WriteInterface;

namespace Borepin.PageModel
{
    public class MachinePageModel : BindableBase, INavigationAware
    {
        private readonly INavigationService _NavigationService;

        public MachinePageModel(INavigationService navigationService)
        {
            _NavigationService = navigationService;

            UseMachineCommand = new DelegateCommand(UseMachineCommandExecuted);
            GiveBackMachineCommand = new DelegateCommand(GiveBackMachineCommandExecuted);
        }

        #region Properties

        private Machine _MachineItem;
        public Machine MachineItem
        {
            get => _MachineItem;
            set => SetProperty(ref _MachineItem, value);
        }

        private string _Name;
        public string Name
        {
            get => _Name;
            set => SetProperty(ref _Name, value);
        }

        private IGiveBack _GiveBack;
        //public IGiveBack GiveBack
        //{
        //    get => _GiveBack;
        //    set => SetProperty(ref _GiveBack, value);
        //}

        private bool _CanUse;
        public bool CanUse
        {
            get => _CanUse;
            set => SetProperty(ref _CanUse, value);
        }

        private bool _CanGiveBack;
        public bool CanGiveBack
        {
            get => _CanGiveBack;
            set => SetProperty(ref _CanGiveBack, value);
        }

        #endregion

        #region Commands
        private ICommand _UseMachineCommand;
        public ICommand UseMachineCommand
        {
            get => _UseMachineCommand;
            set => SetProperty(ref _UseMachineCommand, value);
        }

        private async void UseMachineCommandExecuted()
        {
            _GiveBack = await MachineItem.Instance.Use();
            CanGiveBack = _GiveBack != null;

            MachineItem.MInfo = await MachineItem.Instance.GetMInfo();

            CanUse = MachineItem.MInfo.State == FabAccessAPI.Schema.State.free;
        }

        private ICommand _GiveBackMachineCommand;
        public ICommand GiveBackMachineCommand
        {
            get => _GiveBackMachineCommand;
            set => SetProperty(ref _GiveBackMachineCommand, value);
        }

        private async void GiveBackMachineCommandExecuted()
        {
            await _GiveBack.Ret();

            _GiveBack = null;
            CanGiveBack = _GiveBack != null;

            MachineItem.MInfo = await MachineItem.Instance.GetMInfo();

            CanUse = MachineItem.MInfo.State == FabAccessAPI.Schema.State.free;
        }

        #endregion

        #region INavigationAware
        public void OnNavigatedFrom(INavigationParameters parameters)
        {
            
        }

        public void OnNavigatedTo(INavigationParameters parameters)
        {
            MachineItem = parameters["instance"] as Machine;
            
            Name = MachineItem.MInfo.Name;

            CanUse = MachineItem.MInfo.State == FabAccessAPI.Schema.State.free;
            CanGiveBack = _GiveBack != null;
        }
        #endregion
    }
}
